@@ -139,7 +139,7 @@ public final class Generator {
 
         public StringEncoder getStringEncoder() {
             if (stringEncoder == null) {
-                stringEncoder = new StringEncoder(context, getState().asciiOnly(), getState().escapeSlash());
+                stringEncoder = new StringEncoder(context, getState().asciiOnly(), getState().scriptSafe());
             }
             return stringEncoder;
         }
@@ -292,7 +292,7 @@ public final class Generator {
                         buffer.append(delim);
                     }
                     buffer.append(shift);
-                    Handler<IRubyObject> handler = getHandlerFor(runtime, element);
+                    Handler<IRubyObject> handler = (Handler<IRubyObject>) getHandlerFor(runtime, element);
                     handler.generate(session, element, buffer);
                 }
 
@@ -354,7 +354,7 @@ public final class Generator {
                         buffer.append((byte)':');
                         buffer.append(space);
 
-                        Handler<IRubyObject> valueHandler = getHandlerFor(runtime, value);
+                        Handler<IRubyObject> valueHandler = (Handler<IRubyObject>) getHandlerFor(runtime, value);
                         valueHandler.generate(session, value, buffer);
                         session.infectBy(value);
                     }
@@ -362,8 +362,8 @@ public final class Generator {
                 state.decreaseDepth();
                 if (!firstPair[0] && objectNl.length() != 0) {
                     buffer.append(objectNl);
-                    buffer.append(Utils.repeat(state.getIndent(), state.getDepth()));
                 }
+                buffer.append(Utils.repeat(state.getIndent(), state.getDepth()));
                 buffer.append((byte)'}');
             }
         };
@@ -428,7 +428,11 @@ public final class Generator {
         new Handler<IRubyObject>() {
             @Override
             RubyString generateNew(Session session, IRubyObject object) {
-                if (object.respondsTo("to_json")) {
+                if (session.getState().strict()) {
+                    throw Utils.newException(session.getContext(),
+                            Utils.M_GENERATOR_ERROR,
+                            object + " not allowed in JSON");
+                } else if (object.respondsTo("to_json")) {
                     IRubyObject result = object.callMethod(session.getContext(), "to_json",
                               new IRubyObject[] {session.getState()});
                     if (result instanceof RubyString) return (RubyString)result;
